#include <Servo.h>
#include <RFduinoBLE.h>

int front = 0, back = 1, left = 2, right = 3;
Servo motors[4];

float throttle = 0.0;

float yaw = 0.0, pitch = 0.0, roll = 0.0;

float desiredYaw = 0.0;
float desiredPitch = 0.0;
float desiredRoll = 0.0;

float yawGain = 0.0015;
float pitchGain = 0.0015;
float rollGain = 0.0015;

float v[3];

void setup() {
  
  motors[front].attach(5);
  motors[back].attach(4);
  motors[right].attach(6);
  motors[left].attach(3);
  
  motorsOff();

  RFduinoBLE.advertisementData = "quad";
  RFduinoBLE.begin();

  // connect to IMU
  Serial.begin(9600);
  RFduino_ULPDelay( SECONDS(6) );
}

void motorsOff() {
  for (int i = 0; i < 4; i++)
    motors[i].writeMicroseconds(1000);
}

void loop() {
  
  RFduino_ULPDelay( MILLISECONDS(10) );
  
  Serial.write("#f");
  Serial.flush();
  
  while (Serial.available() < 12);
  
  yaw = razorFloat();
  pitch = razorFloat();
  roll = razorFloat();
  
  float dyaw = (desiredYaw - yaw) * yawGain;
  float dpitch = (desiredPitch - pitch) * pitchGain;
  float droll = (desiredRoll - roll) * rollGain;
  
  if (throttle == 0)
    motorsOff();
  else {
    motors[left].writeMicroseconds(mapf(throttle - droll - dyaw, 0.0, 1.0, 1200, 2000));
    motors[right].writeMicroseconds(mapf(throttle + droll - dyaw, 0.0, 1.0, 1200, 2000));
    motors[front].writeMicroseconds(mapf(throttle - dpitch + dyaw, 0.0, 1.0, 1200, 2000));
    motors[back].writeMicroseconds(mapf(throttle + dpitch + dyaw, 0.0, 1.0, 1200, 2000));
  }

}

void RFduinoBLE_onReceive(char *data, int len) {
  if (len >= 5) {
    if (data[0] == 4)
      throttle = bytesToFloat((data + 1));
    if (data[0] == 1)
      yawGain = bytesToFloat((data + 1));
    if (data[0] == 2)
      pitchGain = bytesToFloat((data + 1));
    if (data[0] == 3)
      rollGain = bytesToFloat((data + 1));
  }
}

long mapf(float x, float in_min, float in_max, float out_min, float out_max)
{
  return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
}

float bytesToFloat(char *d) {
  union {
    char c[4];
    float f;
  } u;
  u.c[0] = (uint8_t)d[0];
  u.c[1] = (uint8_t)d[1];
  u.c[2] = (uint8_t)d[2];
  u.c[3] = (uint8_t)d[3];
  return u.f;
}

float razorFloat() {
  char d[4];
  d[0] = (uint8_t)Serial.read();
  d[1] = (uint8_t)Serial.read();
  d[2] = (uint8_t)Serial.read();
  d[3] = (uint8_t)Serial.read(); 
  return bytesToFloat(d);
}

